@@ -2,6 +2,28 @@ if (! window._OQAjaxQueryLoaded)
 (function(){
   window._OQAjaxQueryLoaded = true;
 
+  window.OQnotify = function(thing) {
+    var $oqmsg;
+    if (thing.responseText) {
+      $oqmsg = $("<div class=OQmsg />").html(thing.responseText)
+      var $x = $oqmsg.find('.OQmsg');
+      if ($x.length > 0) {
+        $oqmsg = $x; 
+      }
+    }
+    else {
+      $oqmsg = $("<div class=OQmsg />").html(thing);
+    }
+    var $oqnotify = $("<div class=OQnotify><button type=button class=OQnotifyOkBut>close</button></div>");
+    $oqnotify.prepend($oqmsg);
+    $("div.OQnotify").remove();
+    $oqnotify.appendTo('body').delay(6000).fadeOut(function(){ $(this).remove(); });
+  };
+
+  $(document).delegate(".OQnotifyOkBut", "click", function(){
+    $(this).closest('.OQnotify').remove();  
+  });
+
   // show column panel when clicked
   $(document).delegate('table.OQdata thead td', 'click', function(e) {
     var $t = $(this);
@@ -139,7 +161,7 @@ if (! window._OQAjaxQueryLoaded)
   var downloadCtr = 0;
   $(document).delegate('.OQDownloadCSV,.OQDownloadHTML,.OQDownloadXML','click', function(){
     var target = 'download'+(downloadCtr++); 
-    var $f = $(this).closest('form');
+    var $f = $('.OQform');
     var $panel = $(this).closest('.OQToolsPanel');
     var $newform = $('<form>').css('display','none').attr({
       action: $f.attr('action'), method: 'POST', target: target });
@@ -160,21 +182,28 @@ if (! window._OQAjaxQueryLoaded)
     $panel.find('.OQToolsCancelBut').click();
   });
 
+  $(document).delegate('.OQToolsCancelBut','click', function(){
+    $(".OQToolsPanel").hide();
+  });
+
+  $(document).delegate('.OQToolsBut','click', function(){
+    $(".OQToolsPanel").toggle().filter(":visible").each(function(){ this.scrollIntoView(true); });
+  });
+
   $(document).delegate('.OQDeleteSavedSearchBut','click', function(){
     var $but = $(this);
-    var $a = $but.prev('a');
-    var $f = $but.closest('form');
+    var $tr = $(this).closest('tr');
+    var id = $tr.find('[data-id]').attr('data-id');
+    var $f = $('.OQform');
     $.ajax({ url: $f[0].action, type: 'POST', dataType: 'json',
       data: {
-        OQdeleteSavedSearch: $a.attr('data-id'),
+        OQdeleteSavedSearch: id,
         module: 'InteractiveQuery2Tools'
       },
       complete: function(jqXHR) {
         if (/report\ deleted/.test(jqXHR.responseText)) {
-          if ($but.siblings('button').length==0)
-            $but.parent().remove();
-          else
-            $but.add($a).remove();
+          OQnotify("report deleted"); 
+          $tr.remove();
         }
         else
           alert('Could not delete report.');
@@ -183,26 +212,12 @@ if (! window._OQAjaxQueryLoaded)
     return false;
   });
 
-  $(document).delegate('.OQLoadSavedSearch', 'click', function() {
-    var $t = $(this);
-    var $f = $t.closest('form');
-    $.ajax({ url: $f[0].action, type: 'POST', dataType: 'json',
-      data: {
-        OQgetSavedSearch: $t.attr('data-id'),
-        module: 'InteractiveQuery2Tools',
-        on_select: $f[0].on_select.value
-      },
-      success: function(x) { refreshDataGrid($f,x); }
-    });
-    return false;
-  });
-
   $(document).delegate('.OQSaveReportBut', 'click', function(e) {
-    var $panel = $(this).closest('.OQToolsPanel');
-    var $f = $panel.closest('form');
+    var $f = $('.OQform');
     var dat = buildParamMap($f);
     dat.module = 'InteractiveQuery2Tools';
-    dat.OQsaveSearchTitle = $.trim($(this).parent().find('input').val()); 
+    var $inp = $(this).parent().find('input');
+    dat.OQsaveSearchTitle = $.trim($inp.val());
     if (! dat.OQsaveSearchTitle) {
       alert('enter a name');
       return true;
@@ -210,13 +225,71 @@ if (! window._OQAjaxQueryLoaded)
     $.ajax({ url: $f[0].action, type: 'POST', data: dat, dataType: 'html',
       complete: function(jqXHR) {
         if (! /report\ saved/.test(jqXHR.responseText))
-          alert('Could not save report.');
+          OQnotify("Could not save report.");
+        else {
+          $inp.val(''); 
+          $("h3[data-toolkey=loadreport]").click();
+          OQnotify("report saved");
+        }
       }
     });
-    $panel.find('.OQToolsCancelBut').click();
     return true;
   });
-  
+
+  $(document).delegate('.OQEmailMergePreviewBut, .OQEmailMergeSendEmailBut', 'click', function(e) {
+    var $f = $('.OQform');
+    var dat = buildParamMap($f);
+    dat.module = 'InteractiveQuery2Tools';
+    dat.tool = 'emailmerge';
+    dat.rows_page = 'All';
+    dat.act = $(this).hasClass('OQEmailMergeSendEmailBut') ? 'execute' : 'preview';
+    $(".OQemailmergeform").find('input,textarea').each(function(){
+      if (this.name && this.value) dat[this.name] = this.value;
+    });
+    $.ajax({ url: $f[0].action, type: 'POST', data: dat, dataType: 'html',
+      complete: function(jqXHR) {
+        var $x = $("<div />").html(jqXHR.responseText).find('.OQemailmergeview');
+        if ($x.length == 0) OQnotify(jqXHR);
+        else {
+          $(".OQemailmergetool").children().hide();
+          $(".OQemailmergetool").append($x);
+        }
+      }
+    });
+    return true;
+  });
+
+  $(document).delegate('.OQEmailMergeDeleteAutoActionBut', 'click', function(e) {
+    var $f = $('.OQform');
+    var dat = {};
+    dat.module = 'InteractiveQuery2Tools';
+    dat.tool = 'emailmerge';
+    dat.act = 'deleteautoaction';
+    dat.id = $("input[name=oq_autoaction_id]").val();
+    $.ajax({ url: $f[0].action, type: 'POST', data: dat, dataType: 'html', complete: OQnotify });
+    return true;
+  });
+
+  $(document).delegate('.OQEmailMergeSaveAutoActionBut', 'click', function(e) {
+    var $f = $('.OQform');
+    var dat = buildParamMap($f);
+    dat.module = 'InteractiveQuery2Tools';
+    dat.tool = 'emailmerge';
+    dat.act = 'saveautoaction';
+    dat.filter_descr = $(".OQFilterDescr").children().eq(1).text();
+    $(".OQemailmergeform").find('input,textarea').each(function(){
+      if (this.name && this.value) dat[this.name] = this.value;
+    });
+    $.ajax({ url: $f[0].action, type: 'POST', data: dat, dataType: 'html', complete: OQnotify });
+    return true;
+  });
+
+  $(document).delegate('.OQEmailMergePreviewBackBut', 'click', function(e) {
+    $(".OQemailmergeview").siblings().show();
+    $(".OQemailmergeview").remove();
+    return true;
+  });
+
   $(document).delegate('.OQAddColumnsBut', 'click', function(e) {
     e.preventDefault();
     var $f = $(this).closest('form');
@@ -267,31 +340,23 @@ if (! window._OQAjaxQueryLoaded)
     return true;
   });
 
-  $(document).delegate('.OQToolsCancelBut', 'click', function() {
-    var $f = $(this).closest('form');
-    $f.children('.OQToolsPanel').remove();
-    $f.removeClass('OQToolsMode');
-    return true;
-  });
-
-  $(document).delegate('.OQToolsBut', 'click', function() {
-    var $f = $(this).closest('form');
-    var dat = buildParamMap($f);
-    dat.module = 'InteractiveQuery2Tools';
-    $f.addClass('OQToolsMode');
-    $.ajax({ url: $f[0].action, type: 'POST', data: dat, dataType: 'html',
-      complete: function(jqXHR) {
-        try {
-          var $p = $('<div>').append(jqXHR.responseText).find('.OQToolsPanel');
-          if ($p.length!=1) throw(1);
-          $f.append($p);
-        } catch(e) {
-          alert('Could not load tools panel.');
-          $f.removeClass('OQToolsMode');
-        }
-      }
-    });
-    return true;
+  $(document).delegate('.OQToolExpander', 'click', function() {
+    var $t = $(this);
+    var $content = $t.next('.OQToolContent');
+    if ($content.length==1) {
+      $content.remove();
+      $t.removeClass('opened');
+    } else {
+      $t.siblings('.opened').removeClass('opened');
+      $t.addClass('opened');
+      $t.siblings('.OQToolContent').remove();
+      // load tool content 
+      var $f = $('form.OQform');
+      var dat = buildParamMap($f);
+      dat.module = 'InteractiveQuery2Tools';
+      dat.tool = $t.attr('data-toolkey');
+      $("<div class=OQToolContent />").insertAfter($t).load($f[0].action, dat);
+    }
   });
 
   $(document).delegate('.OQFilterDescr', 'click', function() {
@@ -433,6 +498,7 @@ if (! window._OQAjaxQueryLoaded)
   $(document.body).delegate('.OQselectBut', 'click', function(){
     var f = this.form;
     var args = $(this).attr('data-rv');
+
     if (args=='') return true;
     var A = args.split('~~~');
     if (! f.on_select.value) {
@@ -440,17 +506,23 @@ if (! window._OQAjaxQueryLoaded)
       return true;
     }
     var wo = window.opener2 || window.opener;
+
     var funcName = f.on_select.value.replace(/\,.*/,'');
     var funcRef = wo[funcName];
     if (! funcRef) {
       alert('could not update parent form');
     } else {
+      var opts = /(\~.*)/.test(f.on_select.value) ? RegExp.$1 : "";
       // Ahhh! can't use funcRef.apply with array args because
       // IE <= 7 can't pass Arrays created in one window to another
       funcRef(A[0],A[1],A[2],A[3],A[4],A[5],A[6],A[7],A[8],A[9]);
-      wo.focus();
-      var wc = window.close2 || window.close;
-      wc();
+      if (/\bnoclose\b/.test(opts)) {
+        $(this).fadeOut();
+      } else {
+        wo.focus();
+        var wc = window.close2 || window.close;
+        wc();
+      }
     }
     return true;
   });
@@ -495,6 +567,16 @@ if (! window._OQAjaxQueryLoaded)
     return false;
   });
 
+  $(document).delegate('select.rexp','change', function(){
+    var $textbox = $(this).next();
+    if (this.selectedIndex==0) {
+      $textbox.show().focus();
+    } else {
+      $textbox.val('').hide();
+    }
+    return true;
+  });
+
   $(document).delegate('button.DeleteFilterElemBut','click', function(){
     var $tr = $(this).closest('tr');
     if ($tr.next().length==1) $tr.next().remove();
@@ -556,7 +638,9 @@ if (! window._OQAjaxQueryLoaded)
   var createFilterStr = function($filterpanel){
     var newfilter = '';
     $filterpanel.find('input,select').each(function(){
+      if (this.disabled) return true;
       var val = $(this).val();
+
       if (this.name && /^arg\_/.test(this.name)) {
         var name = this.name.replace(/^arg\_/,'');
         if (/\s/.test(val) || val=='') {
@@ -567,16 +651,18 @@ if (! window._OQAjaxQueryLoaded)
         if (! /\($/.test(newfilter)) newfilter += ',';
         newfilter += name + ',' + val;
       }
-      else if ($(this).hasClass('rexp')) {
+      else if ($(this).is('select.rexp')) {
+        if (val) newfilter += ' '+val;
+      }
+      else if ($(this).is('input.rexp')) {
         if (/\"/.test(val)) val = "'"+val+"'";   
         else val = '"'+val+'"';   
         newfilter += ' '+val;
       }
-      else {
+      else if (val != '') {
         newfilter += ' '+val;
       }
 
-      if (val != '') val += ' AND ';
       return true;
     });
     return $.trim(newfilter);
