@@ -74,7 +74,7 @@ sub execute {
 
   # if clobs have been selected, find & set LongReadLen
   local $sth->{oq}->{dbh}->{'LongReadLen'};
-  if ($$sth{oq}{dbh}{Driver}{Name} eq 'Oracle' &&
+  if ($$sth{oq}{dbtype} eq 'Oracle' &&
       $$sth{'oq'}{'AutoSetLongReadLen'} &&
       scalar(@{$$c{'selected_lobs'}})) {
     my ($SetLongReadLen) = $$sth{oq}{dbh}->selectrow_array("
@@ -87,9 +87,11 @@ sub execute {
   }
 
   eval {
+    $$sth{oq}{error_handler}->("DEBUG SQL:\n".$c->{sql}."\n\nBINDS:\n".join(',', @{ $c->{binds} })."\n\n") if $$sth{oq}{debug};
+    local $sth->{oq}->{dbh}->{FetchHashKeyName} = 'NAME_uc';
     $c->{sth} = $sth->{oq}->{dbh}->prepare($c->{sql});
     $c->{sth}->execute( @{ $c->{binds} } );
-    my @fieldnames = @{ $c->{sth}->{ $sth->{oq}->{dbh}->{FetchHashKeyName} || 'NAME' } };
+    my @fieldnames = @{ $c->{sth}->{ $sth->{oq}->{dbh}->{FetchHashKeyName} || 'NAME_uc' } };
     my %rec;
     my @bindcols = \( @rec{ @fieldnames } );
     $c->{sth}->bind_columns(@bindcols);
@@ -100,7 +102,6 @@ sub execute {
   }
 }
 
-
 # function to add limit sql
 # $sth->add_limit_sql($lo_limit,$hi_limit);
 sub add_limit_sql {
@@ -110,7 +111,7 @@ sub add_limit_sql {
   my $hi_limit = shift or die "missing hi_limit";
   my $c = $sth->{cursors}->[0];
 
-  if ($$sth{oq}{dbh}{Driver}{Name} eq 'Oracle') {
+  if ($$sth{oq}{dbtype} eq 'Oracle') {
     $c->{sql} = "
 SELECT * 
 FROM (
@@ -124,6 +125,46 @@ WHERE tablernk2.RANK >= ? ";
     push @{$$c{binds}}, ( $hi_limit, $lo_limit );
   }
 
+  # sqlserver doesn't support limit/offset until Sql Server 2012 (which I don't have to test)
+  # the workaround is this ugly hack...
+  elsif ($$sth{oq}{dbtype} eq 'Microsoft SQL Server') {
+    die "missing required U_ID in select" unless exists $$sth{oq}{select}{U_ID};
+
+    my $sql = $c->{sql};
+
+    # extract order by sql, and binds in order by from sql
+    my $orderbysql;
+    if ($sql =~ s/\ (ORDER BY\ .*?)$//) {
+      $orderbysql = $1;
+      my $copy = $orderbysql;
+      my $bindCount = $copy =~ tr/,//;
+      if ($bindCount > 0) {
+        my @newBinds;
+        push @newBinds, pop @{$$c{binds}} for 1 .. $bindCount;
+        @{$$c{binds}} = (reverse @newBinds, @{$$c{binds}});
+      }
+      $orderbysql .= ", ".$$sth{oq}{select}{U_ID}[1][0];
+    } elsif (exists $$sth{oq}{select}{U_ID}) {
+      $orderbysql = " ORDER BY ".$$sth{oq}{select}{U_ID}[1][0];
+    }
+
+    # remove first select keyword, and add new one with windowing
+    if ($sql =~ s/^(\s*SELECT\s*)//) {
+      my $limit = int($hi_limit - $lo_limit + 1);
+      my $lo_limit = int($lo_limit);
+
+      # sqlserver doesn't allow placeholders for limit and offset here
+      $c->{sql} = "SELECT TOP $limit * FROM (SELECT ROW_NUMBER() OVER ($orderbysql) AS RANK, $sql) tablerank1 WHERE tablerank1.RANK >= $lo_limit";
+    }
+  }
+
+  elsif ($$sth{oq}{dbtype} eq 'Pg') {
+    my $a = $lo_limit - 1;
+    my $b = $hi_limit - $lo_limit + 1;
+    $c->{sql} .= "\nLIMIT ? OFFSET ?";
+    push @{$$c{binds}}, ($b, $a);
+  }
+
   else {
     my $a = $lo_limit - 1;
     my $b = $hi_limit - $lo_limit + 1;
@@ -253,9 +294,10 @@ sub create_select {
     # if type is date then use specified date format
     if (! $$select[3]{select_sql} && $$select[3]{date_format}) {
       my @tmp = @{ $select->[1] }; $select_sql = \ @tmp; # need a real copy cause we are going to mutate it
-      if ($$sth{oq}{dbh}{Driver}{Name} eq 'Oracle') {
+      if ($$sth{oq}{dbtype} eq 'Oracle' ||
+          $$sth{oq}{dbtype} eq 'Pg') {
         $$select_sql[0] = "to_char(".$$select_sql[0].",'".$$select[3]{date_format}."')";
-      } elsif ($$sth{oq}{dbh}{Driver}{Name} eq 'mysql') {
+      } elsif ($$sth{oq}{dbtype} eq 'mysql') {
         $$select_sql[0] = "date_format(".$$select_sql[0].",'".$$select[3]{date_format}."')";
       } else {
         die "unsupported DB";
@@ -268,7 +310,7 @@ sub create_select {
     }
 
     # remember if a lob is selected
-    if ($$sth{oq}{dbh}{Driver}{Name} eq 'Oracle' &&
+    if ($$sth{oq}{dbtype} eq 'Oracle' &&
         $sth->{oq}->get_col_types('select')->{$show} eq 'clob') {
       push @{ $cursor->{selected_lobs} }, $show;
       #$select_sql->[0] = 'to_char('.$select_sql->[0].')';
@@ -444,9 +486,9 @@ sub create_where {
       # convert rval to a string using date_format
       if (exists $$oq{select}{$token[0]{colAlias}}[3]{date_format} &&     
           $token[1]{sql} =~ /like/i && $token[2]{sql} eq '?') {
-        if ($$oq{dbh}{Driver}{Name} eq 'Oracle') {
+        if ($$oq{dbtype} eq 'Oracle') {
           $token[0]{sql} = "to_char(".$token[0]{sql}.",'".$$oq{select}{$token[0]{colAlias}}[3]{date_format}."')";
-        } elsif ($$oq{dbh}{Driver}{Name} eq 'mysql') {
+        } elsif ($$oq{dbtype} eq 'mysql') {
           $token[0]{sql} = "date_format(".$token[0]{sql}.",'".$$oq{select}{$token[0]{colAlias}}[3]{date_format}."')";
         }  
       }
@@ -456,9 +498,9 @@ sub create_where {
       # convert rval to a date using date_format
       elsif (exists $$oq{select}{$token[0]{colAlias}}[3]{date_format} &&     
           $token[1]{sql} !~ /like/i && $token[2]{sql} eq '?') {
-        if ($$oq{dbh}{Driver}{Name} eq 'Oracle') {
+        if ($$oq{dbtype} eq 'Oracle') {
           $token[2]{sql} = "to_date(?,'".$$oq{select}{$token[0]{colAlias}}[3]{date_format}."')";
-        } elsif ($$oq{dbh}{Driver}{Name} eq 'mysql') {
+        } elsif ($$oq{dbtype} eq 'mysql') {
           $token[2]{sql} = "str_to_date(?,'".$$oq{select}{$token[0]{colAlias}}[3]{date_format}."')";
         }  
       }
@@ -673,7 +715,7 @@ sub create_order_by {
         die "could not find nice name" if $$oq{select}{$colAlias}[2] eq '';
         my $nice = '['.$$oq{select}{$colAlias}[2].']';
 
-        if ($$sth{oq}{dbh}{Driver}{Name} eq 'Oracle' &&
+        if ($$sth{oq}{dbtype} eq 'Oracle' &&
             $sth->{oq}->get_col_types('select')->{$colAlias} eq 'clob' &&
             $sql !~ /^cast\(/i) {
           $sql = "cast($sql as varchar2(100))";
@@ -826,7 +868,7 @@ WHERE $where_sql ";
 
     # if clobs have been selected, find & set LongReadLen
     local $sth->{oq}->{dbh}->{'LongReadLen'};
-    if ($$sth{oq}{dbh}{Driver}{Name} eq 'Oracle' &&
+    if ($$sth{oq}{dbtype} eq 'Oracle' &&
         $$sth{'oq'}{'AutoSetLongReadLen'} &&
         scalar(@{$$c{'selected_lobs'}})) {
       my ($SetLongReadLen) = $$sth{oq}{dbh}->selectrow_array("
@@ -839,6 +881,7 @@ WHERE $where_sql ";
     }
 
 
+    local $sth->{oq}->{dbh}->{FetchHashKeyName} = 'NAME_uc';
     $c->{sth} = $sth->{oq}->{dbh}->prepare($c->{sql});
   }
   $$sth{oq}{error_handler}->("DEBUG: cursors-\n".Dumper($sth->{cursors})."\n") if $$sth{oq}{debug};
@@ -879,7 +922,7 @@ sub fetchrow_hashref  {
         map { $$rec{$_} } @{ $c->{parent_keys} } );
 
       # for some reason when I call fetchrowhashref perl crashes hard
-      my $cols = $c->{sth}->{NAME};
+      my $cols = $c->{sth}->{NAME_uc};
       @$rec{ @$cols } = [];
       while (my @vals = $c->{sth}->fetchrow_array()) {
         for (my $i=0; $i <= $#$cols; $i++) {
@@ -1101,6 +1144,8 @@ sub new {
 
   $oq->_normalize();
 
+  $$oq{dbtype} = $$oq{dbh}{Driver}{Name};
+  $$oq{dbtype} = $$oq{dbh}->get_info(17) if $$oq{dbtype} eq 'ODBC';
 
   return $oq;
 }
@@ -1450,6 +1495,7 @@ sub type_map {
   1 => 'char',
   3 => 'num',    # is decimal type
   4 => 'num',
+  6 => 'num',    # float
   7  => 'num',
   8 => 'num',
   9 => 'date',
@@ -1461,6 +1507,7 @@ sub type_map {
   40 => 'clob',
   91 => 'date',
   93 => 'date',
+  95 => 'date',
   'INTEGER' => 'num',
   'TEXT' => 'char',
   'VARCHAR' => 'char'
@@ -1524,7 +1571,9 @@ sub get_col_types {
         # don't bother though if there is binds
         # this isn't neccessary for mysql since an explicit limit is
         # defined latter
-        push @where, "to_char($sql) = NULL" if $#binds == -1;
+        if ($$oq{dbtype} eq 'Oracle' && $#binds == -1) {
+          push @where, "to_char($sql) = NULL";
+        }
       }
     }
   }
@@ -1574,12 +1623,13 @@ sub get_col_types {
 SELECT ".join(',', @select)."
 FROM $fromSql";
 
-    if ($$oq{dbh}{Driver}{Name} eq 'Oracle') {
+    if ($$oq{dbtype} eq 'Oracle' || $$oq{dbtype} eq 'Microsoft SQL Server') {
       $sql .= "
 WHERE 1=2
 $where ";
     } 
-    elsif ($$oq{dbh}{Driver}{Name} eq 'mysql') {
+
+    elsif ($$oq{dbtype} eq 'mysql') {
       $sql .= "
 LIMIT 0 ";
     }
@@ -1588,6 +1638,7 @@ LIMIT 0 ";
     eval {
       local $oq->{dbh}->{PrintError} = 0;
       local $oq->{dbh}->{RaiseError} = 1;
+      local $oq->{dbh}->{FetchHashKeyName} = 'NAME_uc';
       $sth = $oq->{dbh}->prepare($sql);
       $sth->execute(@binds);
     }; if ($@) {
@@ -1600,7 +1651,7 @@ LIMIT 0 ";
       my $name = $selectColAliasOrder[$i];
       my $type_code = $sth->{TYPE}->[$i];
       my $type = $type_map->{$type_code} or 
-        die "could not find type code: $type_code";
+        die "could not find type code: $type_code for col $name";
       $oq->{'col_types'}->{$selectColTypeOrder[$i]}->{$name} = $type;
 
       # set the type for select, filter, and sort to the default
